@@ -214,15 +214,19 @@ impl Grid {
                 self.size.cols,
                 i,
                 wrapping,
-                prev_pos,
-                prev_attrs,
+                Some(prev_pos),
+                Some(prev_attrs),
             );
             prev_pos = new_pos;
             prev_attrs = new_attrs;
             wrapping = row.wrapped();
         }
 
-        self.write_cursor_position_formatted(contents, Some(prev_pos));
+        self.write_cursor_position_formatted(
+            contents,
+            Some(prev_pos),
+            Some(prev_attrs),
+        );
 
         prev_attrs
     }
@@ -254,7 +258,11 @@ impl Grid {
             wrapping = row.wrapped();
         }
 
-        self.write_cursor_position_formatted(contents, Some(prev_pos));
+        self.write_cursor_position_formatted(
+            contents,
+            Some(prev_pos),
+            Some(prev_attrs),
+        );
 
         prev_attrs
     }
@@ -263,7 +271,9 @@ impl Grid {
         &self,
         contents: &mut Vec<u8>,
         prev_pos: Option<Pos>,
+        prev_attrs: Option<crate::attrs::Attrs>,
     ) {
+        let prev_attrs = prev_attrs.unwrap_or_default();
         // writing a character to the last column of a row doesn't wrap the
         // cursor immediately - it waits until the next character is actually
         // drawn. it is only possible for the cursor to have this kind of
@@ -285,7 +295,9 @@ impl Grid {
                 } else {
                     crate::term::MoveTo::new(pos).write_buf(contents);
                 }
+                cell.attrs().write_escape_code_diff(contents, &prev_attrs);
                 contents.extend(cell.contents().as_bytes());
+                prev_attrs.write_escape_code_diff(contents, cell.attrs());
             } else {
                 // if the cell doesn't have contents, we can't have gotten
                 // here by drawing a character in the last column. this means
@@ -306,17 +318,36 @@ impl Grid {
                     }
                     let cell = self.visible_cell(pos).unwrap();
                     if cell.has_contents() {
+                        // not sure why this is triggering here, seems like a
+                        // clippy bug
+                        #[allow(clippy::option_if_let_else)]
                         if let Some(prev_pos) = prev_pos {
                             if prev_pos.row != i
                                 || prev_pos.col < self.size.cols
                             {
                                 crate::term::MoveFromTo::new(prev_pos, pos)
                                     .write_buf(contents);
+                                cell.attrs().write_escape_code_diff(
+                                    contents,
+                                    &prev_attrs,
+                                );
                                 contents.extend(cell.contents().as_bytes());
+                                prev_attrs.write_escape_code_diff(
+                                    contents,
+                                    cell.attrs(),
+                                );
                             }
                         } else {
                             crate::term::MoveTo::new(pos).write_buf(contents);
+                            cell.attrs().write_escape_code_diff(
+                                contents,
+                                &prev_attrs,
+                            );
                             contents.extend(cell.contents().as_bytes());
+                            prev_attrs.write_escape_code_diff(
+                                contents,
+                                cell.attrs(),
+                            );
                         }
                         contents.extend(
                             "\n".repeat((orig_row - i) as usize).as_bytes(),
@@ -341,10 +372,18 @@ impl Grid {
                         crate::term::MoveTo::new(pos).write_buf(contents);
                     }
                     contents.push(b' ');
+                    // we know that the cell has no contents, but it still may
+                    // have drawing attributes (background color, etc)
+                    let end_cell = self.visible_cell(pos).unwrap();
+                    end_cell
+                        .attrs()
+                        .write_escape_code_diff(contents, &prev_attrs);
                     crate::term::SaveCursor::default().write_buf(contents);
                     crate::term::Backspace::default().write_buf(contents);
                     crate::term::EraseChar::new(1).write_buf(contents);
                     crate::term::RestoreCursor::default().write_buf(contents);
+                    prev_attrs
+                        .write_escape_code_diff(contents, end_cell.attrs());
                 }
             }
         } else if let Some(prev_pos) = prev_pos {
@@ -433,8 +472,7 @@ impl Grid {
         let size = self.size;
         let pos = self.pos;
         let row = self.current_row_mut();
-        for col in pos.col..((pos.col + count).min(size.cols)) {
-            row.clear_wide(col);
+        for col in pos.col..((pos.col.saturating_add(count)).min(size.cols)) {
             row.erase(col as usize, attrs);
         }
     }
@@ -443,6 +481,7 @@ impl Grid {
         for _ in 0..count {
             self.rows.remove(self.scroll_bottom as usize);
             self.rows.insert(self.pos.row as usize, self.new_row());
+            self.rows[self.scroll_bottom as usize].wrap(false);
         }
     }
 
@@ -476,6 +515,7 @@ impl Grid {
         for _ in 0..count {
             self.rows.remove(self.scroll_bottom as usize);
             self.rows.insert(self.scroll_top as usize, self.new_row());
+            self.rows[self.scroll_bottom as usize].wrap(false);
         }
     }
 
@@ -511,11 +551,16 @@ impl Grid {
         self.row_clamp_bottom(in_scroll_region);
     }
 
-    pub fn row_inc_scroll(&mut self, count: u16) {
+    pub fn row_inc_scroll(&mut self, count: u16) -> u16 {
         let in_scroll_region = self.in_scroll_region();
         self.pos.row = self.pos.row.saturating_add(count);
         let lines = self.row_clamp_bottom(in_scroll_region);
-        self.scroll_up(lines);
+        if in_scroll_region {
+            self.scroll_up(lines);
+            lines
+        } else {
+            0
+        }
     }
 
     pub fn row_dec_clamp(&mut self, count: u16) {
@@ -569,9 +614,14 @@ impl Grid {
 
     pub fn col_wrap(&mut self, width: u16, wrap: bool) {
         if self.pos.col > self.size.cols - width {
-            self.current_row_mut().wrap(wrap);
+            let mut prev_pos = self.pos;
             self.pos.col = 0;
-            self.row_inc_scroll(1);
+            let scrolled = self.row_inc_scroll(1);
+            prev_pos.row -= scrolled;
+            let new_pos = self.pos;
+            self.drawing_row_mut(prev_pos)
+                .unwrap()
+                .wrap(wrap && prev_pos.row + 1 == new_pos.row);
         }
     }
 
