@@ -294,10 +294,12 @@ impl Screen {
                 width,
                 i,
                 wrapping,
-                crate::grid::Pos { row: i, col: start },
-                crate::attrs::Attrs::default(),
+                None,
+                None,
             );
-            wrapping = row.wrapped();
+            if start == 0 && width == self.grid.size().cols {
+                wrapping = row.wrapped();
+            }
             contents
         })
     }
@@ -551,10 +553,12 @@ impl Screen {
     /// need to restore the terminal state without the terminal contents
     /// necessarily being the same.
     ///
-    /// Note that this is more complicated than it sounds, because the cursor
-    /// position's state includes more than just the data returned by
-    /// `cursor_position`, since moving the cursor to the next row during text
-    /// wrapping is delayed until a character is written.
+    /// Note that the bytes returned by this function may alter the active
+    /// drawing attributes, because it may require redrawing existing cells in
+    /// order to position the cursor correctly (for instance, in the case
+    /// where the cursor is past the end of a row). Therefore, you should
+    /// ensure to reset the active drawing attributes if necessary after
+    /// processing this data, for instance by using `attributes_formatted`.
     #[must_use]
     pub fn cursor_state_formatted(&self) -> Vec<u8> {
         let mut contents = vec![];
@@ -564,7 +568,15 @@ impl Screen {
 
     fn write_cursor_state_formatted(&self, contents: &mut Vec<u8>) {
         crate::term::HideCursor::new(self.hide_cursor()).write_buf(contents);
-        self.grid().write_cursor_position_formatted(contents, None);
+        self.grid()
+            .write_cursor_position_formatted(contents, None, None);
+
+        // we don't just call write_attributes_formatted here, because that
+        // would still be confusing - consider the case where the user sets
+        // their own unrelated drawing attributes (on a different parser
+        // instance) and then calls cursor_state_formatted. just documenting
+        // it and letting the user handle it on their own is more
+        // straightforward.
     }
 
     /// Returns the `Cell` object at the given location in the terminal, if it
@@ -807,17 +819,6 @@ impl Screen {
 
         let width = c.width().unwrap_or(0).try_into().unwrap();
 
-        // zero width characters still cause the cursor to wrap - this doesn't
-        // affect which cell they go into (the "previous cell" for both (row,
-        // max_col + 1) and (row + 1, 0) is (row, max_col)), but does affect
-        // further movement afterwards - writing an `a` at (row, max_col)
-        // followed by a crlf puts the cursor at (row + 1,
-        // 0), but writing a `à` (specifically `a` followed by a combining
-        // grave accent - the normalized U+00E0 "latin small letter a with
-        // grave" behaves the same as `a`) at (row, max_col) followed by a
-        // crlf puts the cursor at (row + 2, 0)
-        let wrap_width = if width == 0 { 1 } else { width };
-
         // it doesn't make any sense to wrap if the last column in a row
         // didn't already have contents. don't try to handle the case where a
         // character wraps because there was only one column left in the
@@ -827,7 +828,7 @@ impl Screen {
         // (xterm handles this by introducing the concept of triple width
         // cells, which i really don't want to do).
         let mut wrap = false;
-        if pos.col > size.cols - wrap_width {
+        if pos.col > size.cols - width {
             let last_cell = self
                 .drawing_cell(crate::grid::Pos {
                     row: pos.row,
@@ -838,7 +839,7 @@ impl Screen {
                 wrap = true;
             }
         }
-        self.grid_mut().col_wrap(wrap_width, wrap);
+        self.grid_mut().col_wrap(width, wrap);
         let pos = self.grid().pos();
 
         if width == 0 {
@@ -914,7 +915,7 @@ impl Screen {
                         col: drawing_pos.col + 1,
                     })
                     .unwrap();
-                next_cell.clear(attrs);
+                next_cell.set(' ', attrs);
             }
 
             let cell = self.current_cell_mut();
@@ -923,16 +924,22 @@ impl Screen {
             if width > 1 {
                 let pos = self.grid().pos();
                 if self.current_cell().is_wide() {
-                    let next_next_cell = self
-                        .drawing_cell_mut(crate::grid::Pos {
-                            row: pos.row,
-                            col: pos.col + 1,
-                        })
-                        .unwrap();
+                    let next_next_pos = crate::grid::Pos {
+                        row: pos.row,
+                        col: pos.col + 1,
+                    };
+                    let next_next_cell =
+                        self.drawing_cell_mut(next_next_pos).unwrap();
                     next_next_cell.clear(attrs);
+                    if next_next_pos.col == size.cols - 1 {
+                        self.grid_mut()
+                            .drawing_row_mut(next_next_pos)
+                            .unwrap()
+                            .wrap(false);
+                    }
                 }
                 let next_cell = self.current_cell_mut();
-                next_cell.clear(attrs);
+                next_cell.clear(crate::attrs::Attrs::default());
                 next_cell.set_wide_continuation(true);
                 self.grid_mut().col_inc(1);
             }
@@ -1564,7 +1571,7 @@ impl vte::Perform for Screen {
 }
 
 fn canonicalize_params_1(params: &vte::Params, default: u16) -> u16 {
-    let first = params.iter().next().map_or(0, |x| x[0]);
+    let first = params.iter().next().map_or(0, |x| *x.get(0).unwrap_or(&0));
     if first == 0 {
         default
     } else {
@@ -1578,10 +1585,10 @@ fn canonicalize_params_2(
     default2: u16,
 ) -> (u16, u16) {
     let mut iter = params.iter();
-    let first = iter.next().map_or(0, |x| x[0]);
+    let first = iter.next().map_or(0, |x| *x.get(0).unwrap_or(&0));
     let first = if first == 0 { default1 } else { first };
 
-    let second = iter.next().map_or(0, |x| x[0]);
+    let second = iter.next().map_or(0, |x| *x.get(0).unwrap_or(&0));
     let second = if second == 0 { default2 } else { second };
 
     (first, second)
@@ -1592,10 +1599,10 @@ fn canonicalize_params_decstbm(
     size: crate::grid::Size,
 ) -> (u16, u16) {
     let mut iter = params.iter();
-    let top = iter.next().map_or(0, |x| x[0]);
+    let top = iter.next().map_or(0, |x| *x.get(0).unwrap_or(&0));
     let top = if top == 0 { 1 } else { top };
 
-    let bottom = iter.next().map_or(0, |x| x[0]);
+    let bottom = iter.next().map_or(0, |x| *x.get(0).unwrap_or(&0));
     let bottom = if bottom == 0 { size.rows } else { bottom };
 
     (top, bottom)
